/*
COPYRIGHT STATUS:
Dec 1st 2001, Fermi National Accelerator Laboratory (FNAL) documents and
software are sponsored by the U.S. Department of Energy under Contract No.
DE-AC02-76CH03000. Therefore, the U.S. Government retains a  world-wide
non-exclusive, royalty-free license to publish or reproduce these documents
and software for U.S. Government purposes.  All documents and software
available from this server are protected under the U.S. and Foreign
Copyright Laws, and FNAL reserves all rights.

Distribution of the software available from this server is free of
charge subject to the user following the terms of the Fermitools
Software Legal Information.

Redistribution and/or modification of the software shall be accompanied
by the Fermitools Software Legal Information  (including the copyright
notice).

The user is asked to feed back problems, benefits, and/or suggestions
about the software to the Fermilab Software Providers.

Neither the name of Fermilab, the  URA, nor the names of the contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

DISCLAIMER OF LIABILITY (BSD):

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED  WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED  WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL FERMILAB,
OR THE URA, OR THE U.S. DEPARTMENT of ENERGY, OR CONTRIBUTORS BE LIABLE
FOR  ANY  DIRECT, INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE  POSSIBILITY OF SUCH DAMAGE.

Liabilities of the Government:

This software is provided by URA, independent from its Prime Contract
with the U.S. Department of Energy. URA is acting independently from
the Government and in its own private capacity and is not acting on
behalf of the U.S. Government, nor as its contractor nor its agent.
Correspondingly, it is understood and agreed that the U.S. Government
has no connection to this software and in no manner whatsoever shall
be liable for nor assume any responsibility or obligation for any claim,
cost, or damages arising out of or resulting from the use of the software
available from this server.

Export Control:

All documents and software available from this server are subject to U.S.
export control laws.  Anyone downloading information from this server is
obligated to secure any necessary Government licenses before exporting
documents or software obtained from this server.
 */
package org.dcache.alarms.types;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.dcache.alarms.AlarmFactory;
import org.dcache.alarms.IAlarms;
import org.dcache.alarms.Severity;
import org.dcache.alarms.dao.AlarmEntry;
import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Test;

/**
 * Some simple correctness tests.
 *
 * @author arossi
 */
public class ChecksumAlarmTest {

    @Test
    public void shouldBeEqualForSameFileEvenIfChecksumsChange()
                    throws JSONException {
        String pnfsId = UUID.randomUUID().toString();
        String storedCsm = UUID.randomUUID().toString();
        String currentCsm1 = UUID.randomUUID().toString();
        String currentCsm2 = UUID.randomUUID().toString();
        String alarm1 = AlarmFactory.givenChecksumMessageForThisHostWith(
                        ChecksumAlarmTest.class.getName(),
                        pnfsId, storedCsm, currentCsm1);


        /*
         * Need identical alarm with different timestamp (autogenerated when
         * alarm is initialized).
         */
        try {
            Thread.sleep(TimeUnit.SECONDS.toMillis(1));
        } catch (InterruptedException ignored) {
        }

        String alarm2 = AlarmFactory.givenChecksumMessageForThisHostWith(
                        ChecksumAlarmTest.class.getName(),
                        pnfsId, storedCsm, currentCsm2);


        assertThat(getKey(alarm1).equals(getKey(alarm2)), is(true));
    }

    private String getKey(String jsonFormat) throws JSONException {
        JSONObject obj = new JSONObject(jsonFormat);
        return obj.getString(IAlarms.KEY_TAG);
    }

    @Test
    public void shouldNotBeEqualForDifferentFilesEvenIfChecksumsSame()
                    throws JSONException {
        String pnfsId1 = UUID.randomUUID().toString();
        String pnfsId2 = UUID.randomUUID().toString();
        String storedCsm = UUID.randomUUID().toString();
        String currentCsm = UUID.randomUUID().toString();
        String alarm1 = AlarmFactory.givenChecksumMessageForThisHostWith(
                        ChecksumAlarmTest.class.getName(),
                        pnfsId1, storedCsm, currentCsm);
        String alarm2 = AlarmFactory.givenChecksumMessageForThisHostWith(
                        ChecksumAlarmTest.class.getName(),
                        pnfsId2, storedCsm, currentCsm);
        assertThat(getKey(alarm1).equals(getKey(alarm2)), is(false));
    }

    @Test
    public void shouldGenerateAlarmEntryWithSameProperties()
                    throws JSONException {
        String pnfsId = UUID.randomUUID().toString();
        String storedCsm = UUID.randomUUID().toString();
        String currentCsm = UUID.randomUUID().toString();
        String alarm = AlarmFactory.givenChecksumMessageForThisHostWith(
                        ChecksumAlarmTest.class.getName(),
                        pnfsId, storedCsm, currentCsm);
        JSONObject json = new JSONObject(alarm);
        AlarmEntry entry = AlarmFactory.givenAlarmEntryFromChecksumMessage(alarm);
        assertThat(entry.getKey(), is(json.getString(IAlarms.KEY_TAG)));
        assertThat(entry.getType(), is(json.getString(IAlarms.TYPE_TAG)));
        assertThat(Severity.fromOrdinal(entry.getSeverity()).toString(),
                        is(json.getString(IAlarms.SEVERITY_TAG)));
        assertThat(entry.getTimestamp().toString(),
                        is(json.getString(IAlarms.TIMESTAMP_TAG)));
        assertThat(entry.getHost(), is(json.getString(IAlarms.HOST_TAG)));
        assertThat(entry.getDomain(), is(json.getString(IAlarms.DOMAIN_TAG)));
        assertThat(entry.getService(), is(json.getString(IAlarms.SERVICE_TAG)));
        assertThat(entry.getInfo(), is(json.getString(IAlarms.MESSAGE_TAG)));
    }
}
